""" version which can be consumed from within the module """
VERSION_STR = "0.0.76"
DESCRIPTION = "pytconf handles configuration and command line parsing"
APP_NAME = "pytconf"
LOGGER_NAME = "pytconf"
